@@ -74,7 +74,7 @@
   #:use-module ((nyacc lex) #:select (cnumstr->scm))
   #:use-module ((nyacc util) #:select (ugly-print))
   #:re-export (*nyacc-version*)
-  #:version (2 00 6))
+  #:version (2 01 0))
 
 ;; maybe change to a record-type
 (define *options* (make-parameter '()))
@@ -249,8 +249,8 @@
 (define (w/union name) (cons 'union name))
 (define (w/enum name) (cons 'enum name))
 (define (w/* name) (cons 'pointer name))
-(define (w/struct* name) (cons 'pointer (cons 'struct name)))
-(define (w/union* name) (cons 'pointer (cons 'union name)))
+(define (w/struct* name) (cons 'struct-pointer name))
+(define (w/union* name) (cons 'union-pointer name))
 
 (define (rename name)
   ((*renamer*) name))
