@@ -35,9 +35,10 @@
   #:use-module (ice-9 format)
   #:use-module (ice-9 match)
   #:use-module (ice-9 popen)
+  #:use-module (ice-9 pretty-print)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
-  #:use-module (ice-9 pretty-print)
+  #:use-module (ice-9 vlist)
   #:use-module (system base language)
   #:use-module (system foreign)
   #:use-module ((system base compile) #:select (compile-file))
@@ -62,7 +63,7 @@
   #:use-module ((nyacc lex) #:select (cnumstr->scm))
   #:use-module ((nyacc util) #:select (ugly-print))
   #:re-export (*nyacc-version*)
-  #:version (2 00 6))
+  #:version (2 01 0))
 
 ;; maybe change to a record-type
 (define *options* (make-parameter '()))
@@ -75,13 +76,22 @@
 
 (define *udict* (make-parameter '()))	   ; udecl dict
 (define *ddict* (make-parameter '()))	   ; cpp-def based dict
-(define *defined* (make-parameter '()))    ; defined by define-fh-...
 
 (define *errmsgs* (make-parameter '()))	; list of warnings
 
+(define *defined* (make-parameter (alist->vhash '())))
 
 ;; === utilities
 
+;; like cons* but intended for use w/ *defined*
+(define-syntax dcons
+  (syntax-rules ()
+    ((_ last) last)
+    ((_ item1 rest ...) (vhash-cons item1 #t (dcons rest ...)))))
+
+;; like member but intended for use w/ *defined*
+(define (dmem? item table) (vhash-assoc item table))
+
 (define-syntax xcons*
   (syntax-rules ()
     ((_ tail) tail)
@@ -237,11 +247,11 @@
 (define (w/union name) (cons 'union name))
 (define (w/enum name) (cons 'enum name))
 (define (w/* name) (cons 'pointer name))
-(define (w/struct* name) (cons 'pointer (cons 'struct name)))
-(define (w/union* name) (cons 'pointer (cons 'union name)))
+(define (w/struct* name) (cons 'struct-pointer name))
+(define (w/union* name) (cons 'union-pointer name))
 
-(define (rename name)
-  ((*renamer*) name))
+(define (rename name ctxt)
+  ((*renamer*) name ctxt))
 
 (define (const-expr->number expr)
   (catch 'c99-error
@@ -264,7 +274,7 @@
   (define uq 'unquote)
   (map
    (lambda (dent)
-     (let* ((name (car dent)) (udecl (cdr dent))
+     (let* ((name (rename (car dent) 'field)) (udecl (cdr dent))
 	    (mdecl (udecl->mdecl (udecl-rem-type-qual udecl)))
 	    (type (expand-tail (md-tail mdecl))))
        (if (and (pair? type) (eq? 'cbitfield (car type)))
@@ -277,10 +287,11 @@
   (map (lambda (defn)
          (sx-match defn
            ((enum-defn (ident ,name) (fixed ,value))
-            (list (string->symbol name) (string->number value)))))
+            (list (string->symbol (rename name 'enum))
+                  (string->number value)))))
        (sx-tail enum-def-list)))
 
-(define def-defined
+(define def-def-list
   '("void" "float" "double" "short" "short int" "signed short"
     "signed short int" "int" "signed" "signed int" "long" "long int"
     "signed long" "signed long int" "long long" "long long int"
@@ -292,6 +303,8 @@
     "uint64_t" "float _Complex" "double _Complex" "char" "signed char"
     "unsigned char" "wchar_t" "char16_t" "char32_t" "_Bool" "bool"))
 
+(define def-defined (alist->vhash (map (lambda (n) (cons n #t)) def-def-list)))
+
 (define ffi-long-long
   (case (sizeof-basetype 'long-long)
     ((8) 'ffi:int64)
@@ -337,7 +350,8 @@
     ("unsigned long long int" . ,ffi-unsigned-long-long)
     ("_Bool" . ffi:int8) ("bool" . ffi:int8)))
 
-(define ffi-defined (map car ffi-typemap))
+(define ffi-defined
+  (alist->vhash (map (lambda (p) (cons (car p) #t)) ffi-typemap)))
 
 (define ffi-symmap
   `((ffi:void . ,void) (ffi:float . ,float) (ffi:double . ,double)
@@ -381,8 +395,8 @@
   (let ((defined (*defined*)))
     (match mtail
       (`((pointer-to) (typename ,name))
-       (let ((name (rename name)))
-	 (if (member (w/* name) defined)
+       (let ((name (rename name 'type)))
+	 (if (dmem? (w/* name) defined)
 	     (strings->symbol name "*")
              `(cpointer ,(mtail->ctype (cdr mtail))))))
       (`((pointer-to) (void))
@@ -402,7 +416,7 @@
       (`((pointer-to) (pointer-to) (function-returning . ,rest) . ,rest)
        `(cpointer (cbase 'void*)))
       (`((pointer-to) (struct-ref (ident ,name)))
-       (if (member (w/struct name) defined)
+       (if (dmem? (w/struct name) defined)
            `(cpointer ,(strings->symbol "struct-" name))
 	   `(cpointer 'void)))
       (`((pointer-to) . ,rest)
@@ -417,11 +431,11 @@
       (__
        (sx-match (car mtail)
          ((typename ,name)
-          (let ((name (rename name)))
+          (let ((name (rename name 'type)))
 	    (cond
              ((member name base-type-name-list)
               `(cbase ',(cstrnam->symnam name)))
-             ((member name defined) (strings->symbol name))
+             ((dmem? name defined) (strings->symbol name))
              (else (let* ((udecl `(udecl (decl-spec-list (type-spec . ,mtail))
                                          (init-declr (ident "_"))))
                           (xdecl (expand-typerefs udecl (*udict*) defined))
@@ -587,18 +601,18 @@
       (`(void) #f)
       (`(typename ,name)
        (cond
-	((member name def-defined) `(unwrap-number ,mname))
-	((member name defined) (defined-type-unwrapper name mname))
+	((dmem? name def-defined) `(unwrap-number ,mname))
+	((dmem? name defined) (defined-type-unwrapper name mname))
 	(else #f)))
       (`(enum-def (ident ,name) ,_)
        (cond
-	((member (w/enum name) defined)
+	((dmem? (w/enum name) defined)
          (list (strings->symbol "unwrap-enum-" name) mname))
 	(else `(unwrap~enum ,mname))))
       (`(enum-def ,_) `(unwrap~enum ,mname))
       (`(enum-ref (ident ,name))
        (cond
-	((member (w/enum name) defined)
+	((dmem? (w/enum name) defined)
          (list (strings->symbol "unwrap-enum-" name) mname))
 	(else `(unwrap~enum ,mname))))
       (`(struct-ref (ident ,name)) `(cdata&-ref ,mname))
@@ -633,33 +647,33 @@
       (`((void)) #f)
       (`((typename ,name))
        (cond
-        ((member name def-defined) #f)
-	((member name defined) (defined-type-wrapper name mname))
+        ((dmem? name def-defined) #f)
+	((dmem? name defined) (defined-type-wrapper name mname))
 	(else #f)))
       (`((enum-def (ident ,name) ,rest))
-       (and (member (w/enum name) defined)
+       (and (dmem? (w/enum name) defined)
             (list (sfsym "wrap-enum-~A" name) mname)))
       (`((enum-def ,_)) #f)
       (`((enum-ref (ident ,name)))
        (cond
-        ((member (w/enum name) defined) (sfsym "wrap-enum-~A" name))
+        ((dmem? (w/enum name) defined) (sfsym "wrap-enum-~A" name))
 	(else 'wrap-enum)))
       (`((pointer-to) (typename ,tname))
        (cond
-        ((member tname ffi-defined) #f)
-	((member (w/* tname) defined)
+        ((dmem? tname ffi-defined) #f)
+	((dmem? (w/* tname) defined)
          (let ((sname (sfsym "~A*" tname)))
            `(make-cdata ,sname ,mname)))
 	(else #f)))
       (`((pointer-to) (struct-ref (ident ,aggr-name) . ,rest))
        (cond
-        ((member (w/struct aggr-name) defined)
+        ((dmem? (w/struct aggr-name) defined)
          (let ((sname (sfsym "struct-~A*" aggr-name)))
          `(make-cdata ,sname ,mname)))
 	(else #f)))
       (`((pointer-to) (union-ref (ident ,aggr-name) . ,rest))
        (cond
-	((member (w/union aggr-name) defined)
+	((dmem? (w/union aggr-name) defined)
          (let ((sname (sfsym "union-~A*" aggr-name)))
          `(make-cdata ,sname ,mname)))
 	(else #f)))
@@ -765,7 +779,8 @@
 (define (cnvt-fctn name return params seed)
   ;; can't use function*-wraps just because of the delay :(
   (define varargs? (and (pair? params) (equal? (last params) '(ellipsis))))
-  (let* ((params names (setup-function return params))
+  (let* ((rname (rename name 'function))
+         (params names (setup-function return params))
          (decl-ret decl-par (function-decls return params))
          (exec-ret exec-par (function-execs return params))
          (urap-par (fold-right (lambda (n u s) (if u (cons `(,n ,u) s) s))
@@ -773,7 +788,7 @@
          (call `((force ~proc) ,@names))
 	 (va-call `(apply ~proc ,@names (map cdr ~rest))))
     (xcons* seed
-      `(define-public ,(string->symbol name)
+      `(define-public ,(string->symbol rname)
          ,(if varargs?
               `(lambda (,@names . ~rest)
 	         (let ((~proc (ffi:pointer->procedure
@@ -806,7 +821,6 @@
 	     (cons (expand-typerefs udecl udict defined) seed))
 	   '() (clean-and-unitize-fields (sx-tail field-list))))))
 
-
 ;; @deffn {Procedure} udecl->sexp udecl udict defined seed)
 ;; Given @var{udecl} produce scheme FFI wrappers for C types, C functions,
 ;; and C variables. Return updated @var{defined}, a string based list of
@@ -854,7 +868,7 @@
      ((memq 'typedef sspec)
       (let* ((specl `(decl-spec-list (type-spec ,tspec)))
              (mdecl (udecl->mdecl (sx-list 'udecl #f specl declr)))
-             (name (rename (md-label mdecl)))
+             (name (rename (md-label mdecl) 'type))
              (type (strings->symbol name))
              (type* (strings->symbol name "*"))
              (mtail (md-tail mdecl)))
@@ -863,7 +877,7 @@
 
           (`((array-of ,dim) . ,rest)
            (values
-            (cons name defined)
+            (dcons name defined)
             (xcons* seed
               (deftype type (mtail->ctype mtail))
               (deftype type* `(cpointer ,type)))))
@@ -873,7 +887,7 @@
              (call-with-values (lambda () (function*-wraps return params))
                (lambda (ptr->proc proc->ptr)
                  (values
-                  (cons* name (w/* name) defined)
+                  (dcons name (w/* name) defined)
                   (xcons* seed
                     (deftype type `(cfunction ,ptr->proc ,proc->ptr))
                     (deftype type* `(cpointer ,type))))))))
@@ -884,7 +898,7 @@
                (let ((*type (strings->symbol "*" name)))
                  (lambda (pr->pc pc->pr)
                    (values
-                    (cons name defined)
+                    (dcons name defined)
                     (xcons* seed
                       `(define ,*type (cfunction ,pr->pc ,pc->pr))
                       (deftype type `(cpointer ,*type)))))))))
@@ -893,7 +907,7 @@
            (values
             (case (caar rest)
               ((fixed-type float-type) defined)
-              (else (cons name defined)))
+              (else (dcons name defined)))
             (xcons* seed
               (deftype type (mtail->ctype mtail)))))
 
@@ -901,9 +915,9 @@
            (sx-match (car mtail)
              
              ((struct-def (@ . ,attr) (ident ,agname) ,field-list)
-              (let ((agname (rename agname)))
+              (let ((agname (rename agname 'type)))
                 (values
-                 (cons* name (w/* name) (w/struct agname)
+                 (dcons name (w/* name) (w/struct agname)
                         (w/struct* agname) defined)
                  (let ((aname (strings->symbol "struct-" agname))
                        (aname* (strings->symbol "struct-" agname "*")))
@@ -915,15 +929,15 @@
 
              ((struct-def ,field-list)
               (values
-               (cons* name (w/* name) defined)
+               (dcons name (w/* name) defined)
                (xcons* seed
                  (deftype type (mtail->ctype mtail))
                  (deftype type* `(cpointer ,type)))))
 
              ((union-def (ident ,agname) ,field-list)
-              (let ((agname (rename agname)))
+              (let ((agname (rename agname 'type)))
                 (values
-                 (cons* name (w/* name) (w/union agname)
+                 (dcons name (w/* name) (w/union agname)
                         (w/union* agname) defined)
                  (let ((aname (strings->symbol "union-" agname))
                        (aname* (strings->symbol "union-" agname "*")))
@@ -935,17 +949,17 @@
 
              ((union-def ,field-list)
               (values
-               (cons* name (w/* name) defined)
+               (dcons name (w/* name) defined)
                (xcons* seed
                  (deftype type (mtail->ctype mtail))
                  (deftype type* `(cpointer ,type)))))
 
              ((struct-ref (ident ,agname))
-              (let ((agname (rename agname)))
+              (let ((agname (rename agname 'type)))
                 (values
-                 (cons* name (w/* name) defined)
+                 (dcons name (w/* name) defined)
                  (cond
-                  ((member (w/struct agname) defined) ;; defined previously
+                  ((dmem? (w/struct agname) defined) ;; defined previously
                    (xcons* seed
                      (deftype type (mtail->ctype mtail))
 	             (deftype type* (sfsym "struct-~A*" agname))))
@@ -961,11 +975,11 @@
                      (deftype type* `(cpointer ,type))))))))
 
              ((union-ref (ident ,agname))
-              (let ((agname (rename agname)))
+              (let ((agname (rename agname 'type)))
                 (values
-                 (cons* name (w/* name) defined)
+                 (dcons name (w/* name) defined)
                  (cond
-                  ((member (w/union agname) defined) ;; defined previously
+                  ((dmem? (w/union agname) defined) ;; defined previously
                    (xcons* seed
                      (deftype type (mtail->ctype mtail))
 	             (deftype type* (sfsym "union-~A*" agname))))
@@ -985,33 +999,33 @@
 
              ((enum-def ,enum-def-list)
               (values
-               (cons name defined)
+               (dcons name defined)
                (xcons* seed
                  (deftype type (mtail->ctype mtail))
                  `(define-public ,(sfsym "unwrap-~A" name)
-                    (let ((vald (cenum-vald (ctype-info ,type))))
-                      (lambda (arg) (or (assq-ref vald arg) arg))))
+                    (let ((numf (cenum-numf (ctype-info ,type))))
+                      (lambda (arg) (or (numf arg) arg))))
                  `(define-public ,(sfsym "wrap-~A" name)
-                    (let ((symd (cenum-symd (ctype-info ,type))))
-                      (lambda (arg) (or (assq-ref symd arg) arg)))))))
+                    (let ((symf (cenum-symf (ctype-info ,type))))
+                      (lambda (arg) (or (symf arg) arg)))))))
 
              ((enum-def (ident ,enum-name) ,enum-def-list)
-              (let ((enum-name (rename enum-name)))
+              (let ((enum-name (rename enum-name 'type)))
                 (values
-                 (cons* name (w/enum enum-name) defined)
+                 (dcons name (w/enum enum-name) defined)
                  (xcons* seed
                    (deftype type (mtail->ctype mtail))
                    `(define-public ,(sfsym "unwrap-~A" name)
-                      (let ((vald (cenum-vald (ctype-info ,type))))
-                        (lambda (arg) (or (assq-ref vald arg) arg))))
+                      (let ((numf (cenum-numf (ctype-info ,type))))
+                        (lambda (arg) (or (numf arg) arg))))
                    `(define-public ,(sfsym "wrap-~A" name)
-                      (let ((symd (cenum-symd (ctype-info ,type))))
-                        (lambda (arg) (or (assq-ref symd arg) arg))))))))
+                      (let ((symf (cenum-symf (ctype-info ,type))))
+                        (lambda (arg) (or (symf arg) arg))))))))
 
              ((enum-ref (ident ,enum-name))
-              (let ((enum-name (rename enum-name)))
+              (let ((enum-name (rename enum-name 'type)))
                 (values
-                 (cons (w/enum enum-name) defined)
+                 (dcons (w/enum enum-name) defined)
                  (xcons* seed
                    (deftype type (sfsym "enum-~a" enum-name))
                    `(define-public ,(sfsym "wrap-~A" name)
@@ -1021,32 +1035,34 @@
 
              ((void)
               (values
-               (cons* name (w/* name) defined)
+               (dcons name (w/* name) defined)
                (xcons* seed
                  `(define-public ,type 'void)
                  (deftype type* `(cpointer ,type)))))
 
              ((typename ,typename)
-              (cond
-	       ((member typename def-defined)
-	        (values defined seed))
-	       ((member (rename typename) defined)
-                (let* ((typename (rename typename))
-                       (aka (string->symbol typename))
-                       (atype (strings->symbol typename))
-                       (defined (cons name defined))
-                       (seed (cons (deftype type atype) seed)))
-	          (if (member (w/* typename) defined)
-                      (let* ((name* (strings->symbol name "*"))
-                             (aka* (strings->symbol typename "*"))
-                             (atype* (strings->symbol typename "*")))
-	                (values
-                         (cons (w/* name) defined)
-                         (cons (deftype type* atype*) seed)))
-                      (values defined seed))))
-	       (else
-	        (let ((xdecl (expand-typerefs udecl (*udict*) defined)))
-	          (udecl->sexp xdecl udict defined seed)))))
+              (let ((typerename (rename typename 'type)))
+                (cond
+	         ((dmem? typename def-defined)
+	          (values defined seed))
+	         ((dmem? typerename defined)
+                  (let* ((typename typerename)
+                         (aka (string->symbol typename))
+                         (atype (strings->symbol typename))
+                         (defined (dcons name defined))
+                         (seed (cons (deftype type atype) seed)))
+	            (if (dmem? (w/* typename) defined)
+                        (let* ((name* (strings->symbol name "*"))
+                               (aka* (strings->symbol typename "*"))
+                               (atype* (strings->symbol typename "*")))
+	                  (values
+                           (dcons (w/* name) defined)
+                           (cons (deftype type* atype*) seed)))
+                        (values defined seed))))
+	        (else
+	         (let ((xdecl (expand-typerefs udecl (*udict*) defined)))
+	           (udecl->sexp xdecl udict defined seed))))))
+             
              (,__
               (sferr "udecl->sexp missed typedef:\n") (pperr mdecl)
               (values defined seed)))))))
@@ -1054,7 +1070,7 @@
      ((ftn-declr? declr)
       (let* ((specl `(decl-spec-list (type-spec ,tspec)))
              (mdecl (udecl->mdecl (sx-list 'udecl #f specl declr)))
-             (name (rename (md-label mdecl)))
+             (name (md-label mdecl))
              (return (mdecl->udecl (cons "~ret" (cdr (md-tail mdecl)))))
              (params (cdadar (md-tail mdecl))))
         (values defined (cnvt-fctn name return params seed))))
@@ -1062,7 +1078,7 @@
      ((sx-match tspec
 
         ((struct-def (@ . ,aggr-attr) (ident ,agname) ,field-list)
-         (let* ((agname (rename agname))
+         (let* ((agname (rename agname 'type))
                 (atype (strings->symbol "struct-" agname))
                 (atype* (strings->symbol "struct-" agname "*"))
                 (field-list (expand-field-list-typerefs field-list))
@@ -1076,16 +1092,16 @@
                (fold-values
 	        (lambda (name defined seed)
                   (let ((type (strings->symbol name)))
-                    (values (cons name defined)
+                    (values (dcons name defined)
                             (cons (deftype type atype) seed))))
 	        name-list
-                (cons (w/struct agname) defined)
+                (dcons (w/struct agname) defined)
                 (xcons* seed
                   (deftype atype agdef)
                   (deftype atype* `(cpointer ,atype))))))
-	    ((not (member (w/struct agname) defined))
+	    ((not (dmem? (w/struct agname) defined))
 	     (values
-              (cons (w/struct agname) defined)
+              (dcons (w/struct agname) defined)
               (xcons* seed
                 (deftype atype agdef)
                 (deftype atype* `(cpointer ,atype)))))
@@ -1093,7 +1109,7 @@
 	     (values defined seed)))))
 
         ((union-def (@ . ,aggr-attr) (ident ,agname) ,field-list)
-         (let* ((agname (rename agname))
+         (let* ((agname (rename agname 'type))
                 (atype (strings->symbol "union-" agname))
                 (atype* (strings->symbol "union-" agname "*"))
                 (field-list (expand-field-list-typerefs field-list))
@@ -1105,16 +1121,16 @@
                (fold-values
 	        (lambda (name defined seed)
                   (let ((type (strings->symbol name)))
-                    (values (cons name defined)
+                    (values (dcons name defined)
                             (cons (deftype type atype) seed))))
 	        name-list
-                (cons (w/union agname) defined)
+                (dcons (w/union agname) defined)
                 (xcons* seed
                   (deftype atype agdef)
                   (deftype atype* `(cpointer ,atype))))))
-	    ((not (member (w/union agname) defined))
+	    ((not (dmem? (w/union agname) defined))
 	     (values
-              (cons (w/union agname) defined)
+              (dcons (w/union agname) defined)
               (xcons* seed
                 (deftype atype agdef)
                 (deftype atype* `(cpointer ,atype)))))
@@ -1122,9 +1138,9 @@
 	     (values defined seed)))))
 
         ((enum-def (ident ,enum-name) ,enum-def-list)
-         (let ((enum-name (rename enum-name)))
+         (let ((enum-name (rename enum-name 'type)))
            (cond
-	    ((member (w/enum enum-name) defined)
+	    ((dmem? (w/enum enum-name) defined)
 	     (values defined seed))
 	    (else
              (let* ((type (sfsym "enum-~a" enum-name))
@@ -1132,15 +1148,15 @@
                            enum-def-list (*udict*) (*ddict*)))
                     (enums (enum-def-list->alist defs)))
 	       (values
-                (cons (w/enum enum-name) defined)
+                (dcons (w/enum enum-name) defined)
                 (xcons* seed
                   (deftype type `(cenum ',enums))
                   `(define-public ,(sfsym "unwrap-~A" type)
-                     (let ((vald (cenum-vald (ctype-info ,type))))
-                       (lambda (arg) (or (assq-ref vald arg) arg))))
+                     (let ((numf (cenum-numf (ctype-info ,type))))
+                       (lambda (arg) (or (numf arg) arg))))
                   `(define-public ,(sfsym "wrap-~A" type)
-                     (let ((symd (cenum-symd (ctype-info ,type))))
-                       (lambda (arg) (or (assq-ref symd arg) arg)))))))))))
+                     (let ((symf (cenum-symf (ctype-info ,type))))
+                       (lambda (arg) (or (symf arg) arg)))))))))))
 
         ((enum-def ,enum-def-list)
 	 (values defined seed))
@@ -1151,6 +1167,7 @@
       (let* ((udecl (expand-typerefs udecl (*udict*) (*defined*)))
              (mdecl (udecl->mdecl udecl))
              (name (md-label mdecl))
+             (rname (rename name 'variable))
              (mtail (cdr (md-tail mdecl))) ; remove (extern)
              (mtail* `((pointer-to) . ,mtail))
              (type* (mtail->ctype mtail*))
@@ -1159,7 +1176,7 @@
          defined
          (xcons* seed
            `(define ,name* ,type*)
-           `(define-public ,(string->symbol name)
+           `(define-public ,(string->symbol rname)
 	      (let* ((obj
                       (delay
                         (make-cdata ,name* (foreign-pointer-search ,name)))))
@@ -1319,7 +1336,7 @@
 	 (cond
 	  ((and ;; Process the declaration if all conditions met:
 	    (declf name)		  ; 1) user wants it
-	    (not (member name defined))	  ; 2) not already defined
+	    (not (dmem? name defined))    ; 2) not already defined
 	    (not (if (pair? name)	  ; 3) not anonymous
 		     (string=? "*anon*" (cdr name))
 		     (string=? "" name))))
@@ -1353,7 +1370,10 @@
 	 (attrs (opts->attrs module-options script-options))
 	 (incf (or (assq-ref attrs 'inc-filter) #f))
 	 (declf (or (assq-ref attrs 'decl-filter) identity))
-	 (renamer (or (assq-ref attrs 'renamer) identity))
+         ;; renamer was of one arg, now two; make it backwards compatible
+	 (renamer (let ((rn (or (assq-ref attrs 'renamer) (lambda (n c) n))))
+                    (if (false-if-exception (rn "foo" 'type))
+                        rn (lambda (n c) (rn n)))))
 	 ;;
 	 (tree (begin
 		 (if (memq 'parse dbugl) (*debug-parse* #t))
@@ -1385,7 +1405,10 @@
 		    (vname (string->symbol (string-append pname "-types")))
 		    (var (module-ref modul vname)))
 	       (append var seed)))
-	   '() ext-mods)))
+	   '() ext-mods))
+         (init-defd (alist->vhash
+                     (map (lambda (n) (cons n #t))
+                          (append-reverse def-def-list ext-defd)))))
 
     ;; set globals
     (*prefix* (m-path->name path))
@@ -1399,18 +1422,13 @@
 
     ;; Convert and output foreign declarations.
     (call-with-values
-	(lambda ()
-	  ;; We need to have externs in wrapped because function parameter
-	  ;; types have wrapped types preserved (e.g., enums).
-	  (process-decls ffi-decls udecls ;; udict <= swap failed 01 Dec 2018
-			 (append def-defined ext-defd) #:declf declf))
+	(lambda () (process-decls ffi-decls udecls init-defd #:declf declf))
       (lambda (defined seed)
 	;; Set ffimod-defined for including, but removed built-in types.
-	(let* ((bity (car def-defined))	; first built-in type
-	       (defd (let loop ((res '()) (defs defined))
-		       (if (eq? (car defs) bity) res
-			   (loop (cons (car defs) res) (cdr defs))))))
-	  (set! ffimod-defined defd))))
+        (set! ffimod-defined
+              (vhash-fold
+               (lambda (k v s) (if (vhash-assoc k init-defd) s (cons k s)))
+               '() defined))))
 
     ;; output global constants (from enum and #define)
     (let* ((modd (c99-trans-unit->ddict tree #:inc-filter incf #:skip-fdefs #t))
