@@ -44,7 +44,8 @@
   #:use-module (srfi srfi-11)           ; let-values
   #:use-module ((srfi srfi-1) #:select (fold fold-right))
   #:use-module (ice-9 match)
-  #:use-module (system base pmatch))
+  #:use-module (ice-9 hash-table)
+  #:use-module (ice-9 vlist))
 
 (use-modules (ice-9 pretty-print))
 (define (sferr fmt . args)
@@ -297,26 +298,17 @@
 (define (w/union name) (cons 'union name))
 (define (w/enum name) (cons 'enum name))
 (define (w/* name) (cons 'pointer name))
-(define (w/struct* name) (cons 'pointer (cons 'struct name)))
-(define (w/union* name) (cons 'pointer (cons 'union name)))
-
-;; qualified match
-;; (qual-match '(pointer struct) "foo" (pointer struct . "foo")) => #t
-(define (qual-match qual name term)
-  (let loop ((qual qual) (term term))
-    (cond
-     ((null? qual)
-      (and (string? term) (string=? name term)))
-     ((pair? term)
-      (and (eq? (car qual) (car term))
-           (loop (cdr qual) (cdr term))))
-     (else #f))))
+(define (w/struct* name) (cons 'struct-pointer name))
+(define (w/union* name) (cons 'union-pointer name))
 
 (define (keeper? qualifier name keepers)
   (cond
+   ((vhash? keepers)
+    (vhash-assoc (if qualifier (cons qualifier name) name) keepers))
+   ((hash-table? keepers)
+    (hash-ref keepers (if qualifier (cons qualifier name) name)))
    ((pair? keepers)
-    (or (qual-match qualifier name (car keepers))
-        (keeper? qualifier name (cdr keepers))))
+    (member (if qualifier (cons qualifier name) name) keepers))
    ((eq? #t keepers) #t)
    (else #f)))
 
@@ -366,10 +358,10 @@
 
       ((typename ,name)
        (cond
-        ((keeper? '() name keep)        ; keeper; don't expand
+        ((keeper? #f name keep)        ; keeper; don't expand
          (values specl declr))
         ((and (pointer-declr? declr)
-              (keeper? '(pointer) name keep))
+              (keeper? 'pointer name keep))
          (values specl declr))
         (else                           ; expand
          (call-with-values
@@ -379,10 +371,10 @@
 
       ((struct-ref (ident ,name))
        (cond
-        ((keeper? '(struct) name keep)
+        ((keeper? 'struct name keep)
          (values specl declr))
         ((pointer-declr? declr)
-         (if (keeper? '(pointer struct) name keep)
+         (if (keeper? 'struct-pointer name keep)
              (values specl declr)
              (values (replace-type-spec specl '(type-spec (void))) declr)))
         (else
@@ -390,10 +382,10 @@
 
       ((union-ref (ident ,name))
        (cond
-        ((keeper? '(union) name keep)
+        ((keeper? 'union name keep)
          (values specl declr))
         ((pointer-declr? declr)
-         (if (keeper? '(pointer union) name keep)
+         (if (keeper? 'union-pointer name keep)
              (values specl declr)
              (values (replace-type-spec specl '(type-spec (void))) declr)))
         (else
@@ -417,7 +409,7 @@
 
       ((enum-ref (@ . ,attr) (ident ,name))
        (cond
-        ((keeper? '(enum) name keep)
+        ((keeper? 'enum name keep)
          (values specl declr))
         ((member '(enum . "*any*") keep)
          (values specl declr))
@@ -428,9 +420,9 @@
       ((enum-def (@ . ,attr) (ident ,name) ,rest)
        ;; replacing with int could be an error : should gen warning
        (cond
-        ((keeper? '(enum) name keep)
+        ((keeper? 'enum name keep)
          (values specl declr))
-        ((member '(enum . "*any*") keep)
+        ((keeper? 'enum "*any*" keep)
          (let ((tspec `(type-spec (enum-ref (ident ,name)))))
            (values (replace-type-spec specl tspec) declr)))
         (else
@@ -439,7 +431,7 @@
 
       ((enum-def (@ . ,attr) ,rest)
        (cond
-        ((member '(enum . "*any*") keep)
+        ((keeper? 'enum "*any*" keep)
          (let ((tspec `(type-spec (enum-ref (ident "*any*")))))
            (values (replace-type-spec specl tspec) declr)))
         (else
